#include <arduinoFFT.h>

template <uint16_t BarNums>
class SoundBars {
public:
  SoundBars(uint8_t DataPin, uint8_t LatchPin, uint8_t ClockPin) 
    : m_DataPin(DataPin), m_LatchPin(LatchPin), m_ClockPin(ClockPin) {
    pinMode(DataPin, OUTPUT);
    pinMode(LatchPin, OUTPUT);
    pinMode(ClockPin, OUTPUT); 
    digitalWrite(m_LatchPin, LOW);
    for (int32_t i = BarNums - 1; i >= 0; i--) {
      shiftOut(m_DataPin, m_ClockPin, MSBFIRST, 0);   
    }
    digitalWrite(m_LatchPin, HIGH);
  }

  void OpeningLights() {
    const uint16_t TimeDelay = 30;
    for (uint8_t k = 0; k <= 4; k++) {
      for (uint8_t i = 1; i <= 8; i++) {
        digitalWrite(m_LatchPin, LOW);
        for (int32_t j = BarNums - 1; j >= 0; j--) {
          shiftOut(m_DataPin, m_ClockPin, MSBFIRST, (1 << i) - 1);   
        }
        digitalWrite(m_LatchPin, HIGH);   
        delay(TimeDelay); 
      }
      for (uint8_t i = 1; i <= 8; i++) {
          digitalWrite(m_LatchPin, LOW);
          for (int32_t j = BarNums - 1; j >= 0; j--) {
            shiftOut(m_DataPin, m_ClockPin, MSBFIRST, (1 << (8 - i)) - 1);   
          }
          digitalWrite(m_LatchPin, HIGH);   
          delay(TimeDelay); 
      }
    }
  }

  void Display(uint8_t RowsLevel[BarNums]) {
    digitalWrite(m_LatchPin, LOW);
    for (int32_t i = BarNums - 1; i >= 0; i--) {
      shiftOut(m_DataPin, m_ClockPin, MSBFIRST, (1 << RowsLevel[i]) - 1);   
    }
    digitalWrite(m_LatchPin, HIGH);
  }
private:
  const uint8_t m_DataPin;
  const uint8_t m_LatchPin;
  const uint8_t m_ClockPin;  
};

SoundBars<4> soundBars(8, 7, 6);

const int micPin = A5;
#define SAMPLES 128
#define SAMPLING_FREQ 40000

const unsigned SAMPLIG_PERIOD_US = round(1000000 * (1.0 / SAMPLING_FREQ));
double fftRealData[SAMPLES];
double fftImgData[SAMPLES];

arduinoFFT FFT(fftRealData, fftImgData, SAMPLES, SAMPLING_FREQ);


void setup() {
  pinMode(micPin, INPUT);
  soundBars.OpeningLights();
}

void loop() {  
  for (int i = 0; i < SAMPLES; i++) {
    int stageTime = micros();
    fftRealData[i] = analogRead(micPin);
    fftImgData[i] = 0;
    while (micros() < (stageTime + SAMPLIG_PERIOD_US));
  }

  FFT.DCRemoval();
  FFT.Windowing(FFT_WIN_TYP_HAMMING, FFT_FORWARD);
  FFT.Compute(FFT_FORWARD);
  FFT.ComplexToMagnitude();

  int bands[4] = {0, 0, 0, 0};

  for (int i = 1; i < SAMPLES / 2; i++) {
    if (i <= 3) {
      bands[0] += (int) fftRealData[i];
    } else if (i <= 10) {
      bands[1] += (int) fftRealData[i];
    } else if (i <= 33) {
      bands[2] += (int) fftRealData[i];
    } else {
      bands[3] += (int) fftRealData[i];
    }
  }
  uint8_t leds[4] = {bands[0]/ 256, bands[1]/ 256, bands[2]/ 256, bands[3]/ 256};
  soundBars.Display(leds);
}
